<?php

namespace Drupal\external_entities\Event;

use Drupal\Component\EventDispatcher\Event;

/**
 * Defines an external entity transliterate filters event.
 *
 * @see \Drupal\external_entities\StorageClient\StorageClientBase::transliterateDrupalFiltersAlter()
 */
class ExternalEntityTransliterateDrupalFiltersEvent extends Event {

  /**
   * The transliterated filter array.
   *
   * @var array
   *
   * @see \Drupal\external_entities\StorageClient\StorageClientInterface::transliterateDrupalFilters()
   */
  protected $transliteratedFilters;

  /**
   * The original filtering parameter array provided.
   *
   * @var array
   */
  protected $originalParameters;

  /**
   * The context.
   *
   * @var array
   */
  protected $context;

  /**
   * Constructs a transform raw data event object.
   *
   * @param array $trans_filters
   *   The transliterated filters.
   * @param array $original_parameters
   *   The original filter parameters.
   * @param array $context
   *   The context.
   */
  public function __construct(
    array $trans_filters,
    array $original_parameters,
    array $context = []
  ) {
    $this->transliteratedFilters = $trans_filters;
    $this->originalParameters = $original_parameters;
    $this->context = $context;
  }

  /**
   * Gets the original filter parameters.
   *
   * @return array
   *   The original filter parameters.
   */
  public function getOriginalParameters() {
    return $this->originalParameters;
  }

  /**
   * Gets the context.
   *
   * @return array
   *   The context.
   */
  public function getContext() {
    return $this->context;
  }

  /**
   * Returns the transliterated filters.
   *
   * @return array
   *   The transliterated filter array.
   *
   * @see \Drupal\external_entities\StorageClient\StorageClientInterface::transliterateDrupalFilters()
   */
  public function getTransliteratedDrupalFilters() {
    return $this->transliteratedFilters;
  }

  /**
   * Sets the new transliterated filters.
   *
   * @param array $trans_filters
   *   The new transliterated filter array.
   *
   * @see \Drupal\external_entities\StorageClient\StorageClientInterface::transliterateDrupalFilters()
   */
  public function setTransliteratedDrupalFilters(array $trans_filters) {
    $this->transliteratedFilters = $trans_filters;
  }

}
